package EDG::RLS;

use strict;
no strict 'refs';
use SOAP::Lite;
use GLite::HTTPS;

our $VERSION = 1.0;

=head1 NAME

EDG::RLS

=head1 DESCRIPTION

EDG::RLS module provides access to some RLS methods through SOAP.

=head1 METHODS

The methods I<die> if there was a SOAP fault.
The fault string and details are added to the I<die> message.

=over 4

=item B<new($URL)>

Creates an RLS object, which is initialized with the
approriate SOAP endpoint. If the URL uses https protocol,
then it will do the HTTPS initialization (locating proxy
certificate) as well.

Note: you have to include the VO name in the URL!

    my $rls = EDG::RLS->new('https://rls.example.org:8443/example');

=cut

sub new
{
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self  = {};
    $self->{SOAP} = undef;
    bless( $self, $class );

    my $url = shift;

    defined $url or 
      print STDERR "RLS endpoint was not defined!\n" and return;

    GLite::HTTPS::setEnv() if $url =~ /^https:/i;
    my $endpoint = "$url/edg-local-replica-catalog/services/edg-local-replica-catalog\n";

    $self->{SOAP} = new SOAP::Lite
	uri => 'urn:edg-local-replica-catalog',
	proxy => "$endpoint",
	on_fault => sub { my($soap, $res) = @_;
			  die $soap->transport->status, "\n" unless(ref $res);
			  die $res->faultcode(),": ",$res->faultstring(),"\n",
                $res->faultdetail(), "\n";
		      };

    return $self;
}

=pod

=item B<addMapping ($GUID, $pfn)>

Add a new mapping to the Catalog.  If there is already a mapping, we
add this mapping to the set of mappings.  If there are no mappings
already, we create a new mapping.

    $rls->addMapping ('GUID...', 'srm://...');

=cut

sub addMapping ($$)
{
	my $self = shift;
	my $guid = shift;
	my $pfn = shift;

    $self->{SOAP}->addMapping($guid, $pfn);
}

=pod

=item B<removeMapping ($GUID, $pfn)>

Remove from the catalog a single guid to pfn mapping.

    $rls->removeMapping ('GUID...', 'srm://...');

=cut

sub removeMapping
{
	my $self = shift;
	my $guid = shift;
	my $pfn = shift;
   	
        $self->{SOAP}->removeMapping($guid, $pfn);
}

=pod 

=item B<$GUID = guidForPfn ($pfn)>

For a given pfn, return the guid that points to it.

The return value will never be I<undef>, because if
the PFN doesn't exist it will die with 'NoSuchPfnException'.

    $guid = $rls->guidForPfn('srm://....');

=cut

sub guidForPfn
{
	my $self = shift;
	my $pfn = shift;
	
	return $self->{SOAP}->guidForPfn($pfn)->result();
}

=pod

=item B<$pfns = getPfns ($GUID)>

For a given guid, return all the physical filenames (reference to a 
string list) we map to in this Catalog.  

This will always have at least one entry, and never be empty.
If the guid does not exist in the catalog, then it will die
with 'NoSuchGuidException'.

    $surls = $rls->getPfns ('GUID....');
    print join(',', @$surls), "\n";

=cut

sub getPfns
{
	my $self = shift;
	my $guid = shift;
	
	return $self->{SOAP}->getPfns($guid)->result();
}


=pod

=back

=head1 SEE ALSO

L<GLite::HTTPS>

=head1 AUTHORS

Peter.Kunszt <Peter.Kunszt@cern.ch>,
Gavin Mccance <Gavin.Mccance@cern.ch>,
Akos.Frohner <Akos.Frohner@cern.ch>

Copyright (c) 2004 CERN, on behalf of the EU EGEE project.
For license conditions see LICENSE file or
L<http://www.edg.org/license.html>.

=cut


1;
